clc
clear
close all
set(0,'defaultTextInterpreter','latex'); %trying to set the default
set(0,'defaultLegendInterpreter','latex')
% set(0,'defaultAxesFontSize',20);
% set(0,'defaultAxesFontName','TimesNewRoman');

%Input data
%Physical parameters
Rg=287.053; %Costante dell'aria (J/kg/K)
mu=1.81e-5; %viscosita` dinamica dell'aria a 20C (Pa s)
pa=101325;
Temp=293;                  %Temperature (K)
rho=pa/(Rg*Temp);
dt=7e-8;                %Time Step (s)
cG=60000/rho;
%Geometrical parameters
R1 =16e-3 ; % inner radius (m)
R2 =33e-3 ;  % outer radius (m)
Rm=(R1+R2)/2;
T1d=0; %deg
T2d=55; %deg
T_tiltd = 25; %deg
T1=T1d*pi/180;
T2=T2d*pi/180;
T_tilt = T_tiltd*pi/180;
% h_max=40e-6;
h_vec = (5:25)*1e-6;
h_min=h_vec(1);
% h_min = 10e-6;
sh=12e-6;

% Lambda=100;
% omega0=Lambda*pa*(h_min/Rm)^2/(6*mu)*1e-3;
% Omega_rpm=(omega0:omega0*0.5:7*omega0)*1e3;
% Omega_rpm = (5:5:100)*1e3;
Omega_rpm = 5e3;
Omega=Omega_rpm*2*pi/60;
no_pad=6;

%Definizione numero elementini nelle due direzioni
M=30; %direzione radiale
N=30;%direzione circonferenziale
deltar=(R2-R1)/(M-1);
deltat=(T2-T1)/(N-1);
nR =linspace(R1,R2,M); %Vettore dei raggi
nT =linspace(T1,T2-deltat,N);%Vettore degli angoli
%caso di periodicit
% nT=[nT(end), nT, nT(1)];
[T,R] = meshgrid(nT,nR) ; %crea le coordinate di tutti i punti %nR lo replica su ogni riga %nT su ogni colonna
% Convert grid to cartesian coordintes
X = R.*cos(T);
Y = R.*sin(T);
%Definisco valore raggio
Rv=sqrt(X.^2+Y.^2);

H0 = ones(M,N)*h_min;
tilt_start = round(N*0.3);
for ind_gap = 1:tilt_start
    ht=h_min+sh*(tilt_start-ind_gap+1)/(tilt_start);
    H0(:,ind_gap)=ht;
end

figure
surf(X*1e3,Y*1e3,H0*1e6)
xlabel('X (mm)'),ylabel('Y (mm)'),zlabel('Air Gap Height ($\mu m$)')
% colorbar
% view(2)
% axis equal
pause(0.1)


%Variable Initialization
%COndizioni Periodicit
% pold=ones(N+2,M)*pa; %N+2 per periodicit circolare lungo theta
% pnew=ones(N+2,M)*pa;
%COndizioni di Dirichelet p=Pa sul bordo esterno

% F_mat = zeros(length(h_vec),length(Omega));
W = zeros(size(Omega));
Qin = W;
Qout = W;

% for omega_ind = 1:length(Omega)
omega = Omega;
for h_i = 1:length(h_vec)
    Pold=ones(M,N)*pa; %MODIFICATO
    Pnew=Pold;

    h_min = h_vec(h_i);
    fprintf('h = %d um\n',h_vec(h_i)*1e6)
    H0(:,:) = h_min;
    tilt_start = round(N*0.3);
    for ind_gap = 1:tilt_start
        ht=h_min+sh*(tilt_start-ind_gap+1)/(tilt_start);
        H0(:,ind_gap)=ht;
    end

    % omega = Omega(omega_ind);
    % fprintf('w = %d krpm\n',Omega_rpm(omega_ind))

    tic
    err_P=10;
    err_F=10;
    err_G=10;

    %portate uscita nodo
    GN=zeros(M,N); %N and M are for BC
    GS=zeros(M,N);
    gO=zeros(M,N);
    gE=zeros(M,N);
    F0=0;
    F1=0;

    iter=0;
    contatore=0;
    toll=1e-5;
    while abs(err_P)>toll || abs(err_G)>toll || abs(err_F)>toll
        iter=iter+1;
        contatore=contatore+1;
        F0=F1;
        Pold=Pnew;


        ii=2:M-1;   jj=2:N-1; %Nodi interni
        %Calcolo portata uscita radiale
        GN(ii,jj)=-(Pold(ii-1,jj).*H0(ii-1,jj).^3.*Rv(ii-1,jj)+Pold(ii,jj).*H0(ii,jj).^3.*Rv(ii,jj))./(24*mu*Rg*Temp*deltar).*(Pold(ii,jj)-Pold(ii-1,jj));
        GS(ii,jj)=-(Pold(ii,jj).*H0(ii,jj).^3.*Rv(ii,jj)+Pold(ii+1,jj).*H0(ii+1,jj).^3.*Rv(ii+1,jj))./(24*mu*Rg*Temp*deltar).*(Pold(ii+1,jj)-Pold(ii,jj));
        %Calcolo portata uscita circonferenziale
        gOp(ii,jj)=-(Pold(ii,jj-1).*H0(ii,jj-1).^3+Pold(ii,jj).*H0(ii,jj).^3)./(12*mu*Rg*Temp*(Rv(ii,jj-1)+Rv(ii,jj)).*deltat).*(Pold(ii,jj)-Pold(ii,jj-1));
        gOc(ii,jj)=omega*(Pold(ii,jj-1).*H0(ii,jj-1).*Rv(ii,jj-1)+Pold(ii,jj).*H0(ii,jj).*Rv(ii,jj))./(4*Rg*Temp);
        gO(ii,jj)=gOp(ii,jj)+gOc(ii,jj);
        gEp(ii,jj)=-(Pold(ii,jj).*H0(ii,jj).^3+Pold(ii,jj+1).*H0(ii,jj+1).^3)./(12*mu*Rg*Temp*(Rv(ii,jj)+Rv(ii,jj+1)).*deltat).*(Pold(ii,jj+1)-Pold(ii,jj));
        gEc(ii,jj)=omega*(Pold(ii,jj).*H0(ii,jj).*Rv(ii,jj)+Pold(ii,jj+1).*H0(ii,jj+1).*Rv(ii,jj+1))./(4*Rg*Temp);
        gE(ii,jj)=gEp(ii,jj)+gEc(ii,jj);
        %Calcolo pressione meato da equazione di continuit
        Pnew(ii,jj)=Pold(ii,jj)+((GN(ii,jj)-GS(ii,jj))*deltat+(gO(ii,jj)-gE(ii,jj))*deltar).*(dt*Rg*Temp)./(H0(ii,jj).*Rv(ii,jj)*deltar*deltat);
        %Calcolo errore Pressione
        err_P=abs(max((Pnew(:)-Pold(:))./Pold(:)));
        % Err_P_i(iter)=err_P;

        %Diverge?
        if any(isnan(Pnew(:))) || any(isinf(Pnew(:)))
            fprintf('Iterations diverge\n');
            return;
        end
        %Calcolo della capacit di carico
        F1=sum((Pnew-pa)*deltar*deltat.*Rv,'all');

        %Calcolo della portata in uscita O-N-E-S
        Gin=sum(GN(ii,jj)*deltat+gO(ii,jj)*deltar,'all');
        Gout=sum(GS(ii,jj)*deltat+gE(ii,jj)*deltar,'all');
        % %Calcolo errore
        err_F=(F1-F0)/F1;
        err_G_num=sum(Gin-Gout,'all');
        err_G=err_G_num/Gout;

        %   if contatore==2000
        %      figure
        %    surf(X*1e3,Y*1e3,Pnew), hold on
        %    xlabel('X (mm)'),ylabel('Y (mm)'),zlabel('Pressure (Pa)')
        %    grid on,set(gca,'FontSize', 24)
        %    contatore=0;
        %   end

    end
    toc
    W(h_i)=F1*no_pad;
    Qin(h_i)=Gin*cG;
    Qout(h_i)=Gout*cG;

    figure
    surf(X*1e3,Y*1e3,Pnew)
    xlabel('X (mm)'),ylabel('Y (mm)'),zlabel('Pressure (Pa)')
    grid on
    pause(0.5)
end
% F_mat(h_i,:) = W;

%% Curve Caratteristiche
% Capacit di carico
figure
plot(h_vec*1e6,W,'b'),hold on
plot(h_vec*1e6,W,'ro'),hold on
xlabel('Air gap $h$ [$\mu m$]')
ylabel('Load Capacity $W$ (N)')
title('Tapered Bearing load capacity at minium rotational speed $\omega_{min}$')
% legend('Numerical','Experimental')
grid on

%% Salvataggio
W_tapered_opt = W;
h_vec_tapered_opt = h_vec;
save Tapered_static_opt W_tapered_opt h_vec_tapered_opt

fid = fopen('sim_data_tapered.txt','wt');
fprintf(fid,'dt: %+.2e\n',dt);
fprintf(fid,'M: %d\nN: %d\n',M,N);
fclose(fid);

%% Plot comp
load Tapered_static_opt.mat
load Tapered_static.mat

figure
hold on
plot(h_vec_tapered*1e6,W_tapered,'-o')
plot(h_vec_tapered_opt*1e6,W_tapered_opt,'-o')
hold off
xlabel('Air gap $h$ [$\mu m$]')
ylabel('Load Capacity $W$ (N)')
title('Tapered Bearing load capacity comparison at $\omega_{min}=5\, krpm$')
legend('Original','Optimized')
grid on




